@@ -82,7 +82,7 @@ import requests
 from packaging.version import parse
 from yarl import URL
 
-__version__ = "0.2.0"
+__version__ = "0.2.2"
 
 
 class VersionWarning(Warning):
@@ -176,7 +176,7 @@ def build_mod(name:str, code:bytes, initial_globals:dict=None, module_path=None,
     return mod
 
 def fail_or_default(default, exception, msg):
-    if default is not Use.mode.nodefault:
+    if default is not Use.mode.fastfail:
         return default
     else:
         raise exception(msg)
@@ -243,7 +243,10 @@ class Use:
     __version__ = __version__  # otherwise setup.py can't find it
     Path = Path
     URL = URL
-    mode = Enum("Mode", "sha256 nodefault")
+    class Hash(Enum):
+        sha256 = hashlib.sha256
+
+    mode = Enum("Mode", "fastfail")
     
     isfunction = inspect.isfunction
     ismethod = inspect.ismethod
@@ -260,19 +263,20 @@ class Use:
     @__call__.register(URL)
     def _use_url(self, 
                 url:URL, 
-                hash_algo:mode=mode.sha256, 
+                hash_algo:Hash=Hash.sha256, 
                 hash_value:str=None, 
                 initial_globals:dict=None, 
                 as_import:str=None,
-                default=mode.nodefault,
+                default=mode.fastfail,
                 aspectize=None,
                 ):
+        assert hash_algo in Use.Hash, f"{hash_algo} is not a valid hashing algorithm!"
+        
         aspectize = aspectize or {}
         response = requests.get(url)
         if response.status_code != 200:
             raise ModuleNotFoundError(f"Could not load {url} from the interwebs, got a {response.status_code} error.")
-        if hash_algo is Use.mode.sha256:
-            this_hash = hashlib.sha256(response.content).hexdigest()
+        this_hash = hash_algo.value(response.content).hexdigest()
         if hash_value:
             if this_hash != hash_value:
                 return fail_or_default(default, UnexpectedHash, f"{this_hash} does not match the expected hash {hash_value} - aborting!")
@@ -295,8 +299,8 @@ To safely reproduce please use hash_algo="{hash_algo}", hash_value="{this_hash}"
                 reloading:bool=False,
                 initial_globals:dict=None, 
                 as_import=None,
-                default=mode.nodefault,
-                aspectize=None):
+                default=mode.fastfail,
+                aspectize=None):  # sourcery skip: remove-redundant-pass
         aspectize = aspectize or {}
         if path.is_dir():
             return fail_or_default(default, ImportError, f"Can't import directory {path}")
@@ -344,9 +348,9 @@ To safely reproduce please use hash_algo="{hash_algo}", hash_value="{this_hash}"
                     version:str=None, 
                     initial_globals:dict=None, 
                     auto_install:bool=False, 
-                    hash_algo:str=mode.sha256, 
+                    hash_algo:str=Hash.sha256, 
                     hash_value:str=None,
-                    default=mode.nodefault,
+                    default=mode.fastfail,
                     aspectize=None,
                     ) -> ModuleType:
 
@@ -360,48 +364,45 @@ To safely reproduce please use hash_algo="{hash_algo}", hash_value="{this_hash}"
 
         # couldn't find any installed package
         if not spec:
-            if auto_install:
-                # TODO: raise appropriate detailed warnings and give helpful info from the json to fix the issue
-                if not (version and hash_value):
-                    raise RuntimeWarning(f"Can't auto-install {name} without a specific version and corresponding hash value")
-
-                response = requests.get(f"https://pypi.org/pypi/{name}/{version}/json")
-                if not response == 200:
-                    return fail_or_default(default, ImportError, f"Tried to auto-install {name} {version} but failed with {response} while trying to pull info from PyPI.")
-                try:
-                    if not response.json()["urls"]:
-                        return fail_or_default(default, ImportError, f"Tried to auto-install {name} {version} but failed because no valid URLs to download could be found.")
-                    else:
-                        for entry in response.json()["urls"]:
-                            url = entry["url"]
-                            that_hash = entry["digests"].get(hash_algo.name)
-                            filename = entry["filename"]
-                            # special treatment?
-                            yanked = entry["yanked"]
-                            if that_hash == hash_value:
-                                break
-                        else:
-                            return fail_or_default(default, ImportError, f"Tried to auto-install {name} {version} but failed because none of the available hashes match the expected hash.")
-                except KeyError:
-                    return fail_or_default(default, ImportError, f"Tried to auto-install {name} {version} but failed because there was a problem with the JSON from PyPI.")
-            
-                with TemporaryDirectory() as directory:
-                    # TODO: chdir etc
-                    # download the file
-                    with open(filename, "wb") as file:
-                        pass
-                    # check the hash
-                    this_hash = securehash_file(file, hash_algo)
-                    if this_hash != hash_value:
-                        return fail_or_default(default, UnexpectedHash, f"Package {name} in temporary {filename} had hash {this_hash}, which does not match the expected {hash_value}, aborting.")
-                    # load it
-                # now that we got something, we can load it
-                spec = importlib.machinery.PathFinder.find_spec(name)
-            
-            # there's nothing installed and no auto-install
-            else:
+            if not auto_install:
                 return fail_or_default(default, ImportError, f"{name} is not installed and auto-install was not requested.")
-        
+
+            # TODO: raise appropriate detailed warnings and give helpful info from the json to fix the issue
+            if not (version and hash_value):
+                raise RuntimeWarning(f"Can't auto-install {name} without a specific version and corresponding hash value")
+
+            response = requests.get(f"https://pypi.org/pypi/{name}/{version}/json")
+            if response != 200:
+                return fail_or_default(default, ImportError, f"Tried to auto-install {name} {version} but failed with {response} while trying to pull info from PyPI.")
+            try:
+                if not response.json()["urls"]:
+                    return fail_or_default(default, ImportError, f"Tried to auto-install {name} {version} but failed because no valid URLs to download could be found.")
+                for entry in response.json()["urls"]:
+                    url = entry["url"]
+                    that_hash = entry["digests"].get(hash_algo.name)
+                    filename = entry["filename"]
+                    # special treatment?
+                    yanked = entry["yanked"]
+                    if that_hash == hash_value:
+                        break
+                else:
+                    return fail_or_default(default, ImportError, f"Tried to auto-install {name} {version} but failed because none of the available hashes match the expected hash.")
+            except KeyError:
+                return fail_or_default(default, ImportError, f"Tried to auto-install {name} {version} but failed because there was a problem with the JSON from PyPI.")
+
+            with TemporaryDirectory() as directory:
+                # TODO: chdir etc
+                # download the file
+                with open(filename, "wb") as file:
+                    pass
+                # check the hash
+                this_hash = securehash_file(file, hash_algo)
+                if this_hash != hash_value:
+                    return fail_or_default(default, UnexpectedHash, f"Package {name} in temporary {filename} had hash {this_hash}, which does not match the expected {hash_value}, aborting.")
+                # load it
+            # now that we got something, we can load it
+            spec = importlib.machinery.PathFinder.find_spec(name)
+
         # now there should be a valid spec defined
 
         # builtins may have no spec, let's not mess with those
@@ -424,7 +425,7 @@ To safely reproduce please use hash_algo="{hash_algo}", hash_value="{this_hash}"
                     )
             except AttributeError:
                 print(f"Cannot determine version for module {name}, continueing.")
-        
+
         return mod
 
 sys.modules["use"] = Use()
